class Card {
    constructor(details) {
        this.systemId = details.system_id;
        this.systemName = details.system_name;
        this.systemFactionId = details.system_faction_id;
        this.systemFactionName = details.system_faction_name;
    }

    render(element, pos) {
        if (!element.length) {
            element = $(' <div id="card_' + pos + '" class="c-example__tilt js-tilt">\n' +
                '        <div class="c-example__tilt-inner"><em></em></div>\n' +
                '        <p></p>\n' +
                '    </div>');
            $('#container').append(element);
        }
        if (element.data('id') !== this.systemId) {
            element.fadeOut().css('background-image', 'url(' + this.getFactionImage(this.systemFactionId, this.systemFactionName) + ')').fadeIn();
            element.find('.c-example__tilt-inner').html(this.systemName).find('p').html(this.systemFactionName);
            element.find('p').html(this.systemFactionName);
            element.data('id', this.systemId);
        }
    }

    refresh(details, element, pos) {
        this.systemId = details.system_id;
        this.systemName = details.system_name;
        this.systemFactionId = details.system_faction_id;
        this.systemFactionName = details.system_faction_name;
        this.render(element, pos);
    }

    getFactionImage(faction_id, faction_name = '') {
        if(faction_id <= 20) {
            return 'https://play.textspaced.com/images/factions/' + faction_id + '.jpg';
        } else {
            return 'https://play.textspaced.com/images/factions/' + faction_name.charAt(0).toLowerCase() + '.jpg';
        }
    }
}

class App {
    constructor(token) {
        this.token = token;
        this.playerId = 0;
        this.currentSystemId = 0;
        this.nearbyList = {};
        this.status = {};
        this.currentSystem = {};
        this.getStatus();
        this.getNearby();
    }

    get(url) {
        let token = this.token;
        return $.ajax({
            url: "https://play.textspaced.com/api/" + url,
            type: 'GET',
            dataType: 'JSON',
            headers: {
                "Authorization": "Bearer " + token
            },
            async: false,
            cache: false
        });
    }

    post(url, params) {
        let token = this.token;
        return $.ajax({
            url: "https://play.textspaced.com/api/" + url,
            type: 'POST',
            dataType: 'JSON',
            headers: {
                "Authorization": "Bearer " + token
            },
            data: params,
            async: false,
            cache: false
        });
    }

    getNearby() {
        if(Object.keys(this.nearbyList).length === 0) {
            this.nearbyList = this.get("lookup/nearby/").responseJSON || {};
        }
        return this.nearbyList;
    }

    getStatus() {
        this.status = this.get("status/").responseJSON || {};
        this.currentSystemId = this.status.system_id;
        return this.status;
    }

    systemDetails(index) {
        let systemDetails = this.nearbyList[index];
        if (Object.keys(systemDetails).length > 0) {
            return {
                'system_id': systemDetails.system_id,
                'system_name': systemDetails.system_name || 'Unnamed',
                'system_faction_id': systemDetails.system_faction_id,
                'system_faction_name': systemDetails.system_faction_name || ''
            }
        } else {
            return {};
        }
    }
}

//define app details
let app;
let current_offset = 3;
let cards = [];
//find offset to current position
//lay out and render the cards on page load
const loadCards = () => {
    for (let i = current_offset-3; i < current_offset+4; ++i) {
        cards[i] = new Card(app.systemDetails(i));
        cards[i].render($('#card_' + i), i);
    }
    tilt();
};

const tilt = () => {
    let small = (window.innerWidth < 1200);
    $('.js-tilt').tilt({
        glare: true,
        maxGlare: .1,
        scale: (small ? 1 : 1.2),
        tiltPerspective: (small ? 500 : 1000),
        tiltScale: 1,
        tiltSpeed: 600,
        tiltMax: (small ? 5 : 10)
    });
};

$(document).on('change', 'input', function (e) {
    $('#container').html('');
    app = new App($(this).val());
    let log = app.getNearby();
    $.each(log, function( key, value ) {
        if(value.system_distance === 0) {
            current_offset = key;
        }
    });
    loadCards(current_offset);
    e.preventDefault();
    return false;
});